@@ -18,7 +18,7 @@ import copy
 import multiprocessing
 import pathlib
 import sys
-from typing import Iterator, List, Tuple
+from typing import Iterator, List, Tuple, cast
 
 from absl import app
 from absl import flags
@@ -34,6 +34,7 @@ from capirca.lib import cisconx
 from capirca.lib import ciscoxr
 from capirca.lib import cloudarmor
 from capirca.lib import gce
+from capirca.lib import gce_vpc_tf
 from capirca.lib import gcp_hf
 from capirca.lib import ipset
 from capirca.lib import iptables
@@ -50,6 +51,7 @@ from capirca.lib import packetfilter
 from capirca.lib import paloaltofw
 from capirca.lib import pcap
 from capirca.lib import policy
+from capirca.lib import sonic
 from capirca.lib import speedway
 from capirca.lib import srxlo
 from capirca.lib import windows_advfirewall
@@ -192,7 +194,9 @@ def RenderFile(base_directory: str, input_file: pathlib.Path,
   nxacl = False
   xacl = False
   paloalto = False
+  sonic_pol = False
   k8s_pol = False
+  gce_vpc_tf_pol = False
 
   try:
     with open(input_file) as f:
@@ -267,10 +271,14 @@ def RenderFile(base_directory: str, input_file: pathlib.Path,
     nft = copy.deepcopy(pol)
   if 'gce' in platforms:
     gcefw = copy.deepcopy(pol)
+  if 'gce_vpc_tf' in platforms:
+    gce_vpc_tf_pol = copy.deepcopy(pol)
   if 'gcp_hf' in platforms:
     gcphf = copy.deepcopy(pol)
   if 'paloalto' in platforms:
     paloalto = copy.deepcopy(pol)
+  if 'sonic' in platforms:
+    sonic_pol = copy.deepcopy(pol)
   if 'cloudarmor' in platforms:
     gca = copy.deepcopy(pol)
   if 'k8s' in platforms:
@@ -399,6 +407,11 @@ def RenderFile(base_directory: str, input_file: pathlib.Path,
       RenderACL(
           str(acl_obj), acl_obj.SUFFIX, output_directory, input_file,
           write_files)
+    if gce_vpc_tf_pol:
+      acl_obj = gce_vpc_tf.TerraformGCE(gce_vpc_tf_pol, exp_info)
+      RenderACL(
+          str(acl_obj), acl_obj.SUFFIX, output_directory, input_file,
+          write_files)
     if gcphf:
       acl_obj = gcp_hf.HierarchicalFirewall(gcphf, exp_info)
       RenderACL(
@@ -410,6 +423,11 @@ def RenderFile(base_directory: str, input_file: pathlib.Path,
       RenderACL(
           str(acl_obj), acl_obj.SUFFIX, output_directory, input_file,
           write_files)
+    if sonic_pol:
+      acl_obj = sonic.Sonic(sonic_pol, exp_info)
+      RenderACL(
+          str(acl_obj), '.json', output_directory, input_file, write_files,
+          True)
     if gca:
       acl_obj = cloudarmor.CloudArmor(gca, exp_info)
       RenderACL(
@@ -432,10 +450,12 @@ def RenderFile(base_directory: str, input_file: pathlib.Path,
       iptables.Error,
       speedway.Error,
       pcap.Error,
+      sonic.Error,
       aclgenerator.Error,
       aruba.Error,
       nftables.Error,
       gce.Error,
+      gce_vpc_tf.Error,
       cloudarmor.Error,
       k8s.Error) as e:
     raise ACLGeneratorError('Error generating target ACL for %s:\n%s' %
@@ -599,7 +619,7 @@ def Run(base_directory: str, definitions_directory: str, policy_file: str,
 
   # thead-safe list for storing files to write
   manager: multiprocessing.managers.SyncManager = context.Manager()
-  write_files: WriteList = manager.list()
+  write_files: WriteList = cast(WriteList, manager.list())
 
   with_errors = False
   logging.info('finding policies...')
