@@ -68,7 +68,6 @@ def IsDefaultDeny(term):
   return True
 
 
-
 def GetNextPriority(priority):
   """Get the priority for the next rule."""
   return priority
@@ -86,6 +85,7 @@ class Term(gcp.Term):
   _TERM_SOURCE_TAGS_LIMIT = 30
   _TERM_TARGET_TAGS_LIMIT = 70
   _TERM_PORTS_LIMIT = 256
+  _TERM_SERVICE_ACCOUNTS_LIMIT = 10
 
   # Firewall rule name has to match specific RE:
   # The first character must be a lowercase letter, and all following characters
@@ -215,6 +215,13 @@ class Term(gcp.Term):
       raise GceFirewallError(
           'GCE firewall rule exceeded number of target tags per rule: %s' %
           self.term.name)
+    if len(
+        self.term.source_service_accounts
+    ) > self._TERM_SERVICE_ACCOUNTS_LIMIT or len(
+        self.term.target_service_accounts) > self._TERM_SERVICE_ACCOUNTS_LIMIT:
+      raise GceFirewallError(
+          'GCE firewall rule exceeded number of service accounts per rule: %s' %
+          self.term.name)
 
     if self.term.source_tag:
       if self.term.direction == 'INGRESS':
@@ -223,6 +230,18 @@ class Term(gcp.Term):
         term_dict['targetTags'] = self.term.source_tag
     if self.term.destination_tag and self.term.direction == 'INGRESS':
       term_dict['targetTags'] = self.term.destination_tag
+    if self.term.source_service_accounts:
+      if 'targetTags' in term_dict or 'sourceTags' in term_dict:
+        raise GceFirewallError(
+            'sourceServiceAccounts cannot be used at the same time as targetTags or sourceTags: %s'
+            % self.term.source_service_accounts)
+      term_dict['sourceServiceAccounts'] = self.term.source_service_accounts
+    if self.term.target_service_accounts:
+      if 'targetTags' in term_dict or 'sourceTags' in term_dict:
+        raise GceFirewallError(
+            'targetServiceAccounts cannot be used at the same time as targetTags or sourceTags: %s'
+            % self.term.target_service_accounts)
+      term_dict['targetServiceAccounts'] = self.term.target_service_accounts
     if self.term.priority:
       term_dict['priority'] = self.term.priority
       # Update term priority for the inet6 rule.
@@ -247,9 +266,11 @@ class Term(gcp.Term):
 
     saddrs = sorted(self.term.GetAddressOfVersion('source_address', term_af),
                     key=ipaddress.get_mixed_type_key)
+    saddrs = gcp.FilterIPv4InIPv6FormatAddrs(saddrs)
     daddrs = sorted(
         self.term.GetAddressOfVersion('destination_address', term_af),
         key=ipaddress.get_mixed_type_key)
+    daddrs = gcp.FilterIPv4InIPv6FormatAddrs(daddrs)
 
     # If the address got filtered out and is empty due to address family, we
     # don't render the term. At this point of term processing, the direction
@@ -388,9 +409,10 @@ class GCE(gcp.GCP):
   # is rendered (which can add proto and a counter).
   _TERM_MAX_LENGTH = 53
   _GOOD_DIRECTION = ['INGRESS', 'EGRESS']
-  _OPTIONAL_SUPPORTED_KEYWORDS = set(['expiration',
-                                      'destination_tag',
-                                      'source_tag'])
+  _OPTIONAL_SUPPORTED_KEYWORDS = frozenset([
+      'expiration', 'destination_tag', 'source_tag', 'source_service_accounts',
+      'target_service_accounts'
+  ])
 
   def _BuildTokens(self):
     """Build supported tokens for platform.
@@ -401,11 +423,10 @@ class GCE(gcp.GCP):
     supported_tokens, _ = super()._BuildTokens()
 
     # add extra things
-    supported_tokens |= {'destination_tag',
-                         'expiration',
-                         'owner',
-                         'priority',
-                         'source_tag'}
+    supported_tokens |= {
+        'destination_tag', 'expiration', 'owner', 'priority', 'source_tag',
+        'source_service_accounts', 'target_service_accounts'
+    }
 
     # remove unsupported things
     supported_tokens -= {'icmp_type',
@@ -582,7 +603,9 @@ def GetAttributeCount(dict_term: Dict[str, Any]) -> int:
     tags += 1
 
   service_accounts = 0
-  for _ in dict_term.get('targetServiceAccount', []):
+  for _ in dict_term.get('sourceServiceAccounts', []):
+    service_accounts += 1
+  for _ in dict_term.get('targetServiceAccounts', []):
     service_accounts += 1
 
   return addresses + proto_ports + tags + service_accounts
