@@ -140,6 +140,7 @@ class Term(aclgenerator.Term):
              'next': 'next term',
              'reject-with-tcp-rst': 'reject tcp-reset',
              'encapsulate': 'encapsulate',
+             'decapsulate': 'decapsulate',
              'port-mirror': 'port-mirror'}
 
   # the following lookup table is used to map between the various types of
@@ -452,18 +453,6 @@ class Term(aclgenerator.Term):
       if self.term.ttl and self.term_type == 'inet':
         config.Append('ttl %s;' % self.term.ttl)
 
-      # protocol
-      if self.term.protocol:
-        # both are supported on JunOS, but only icmp6 is supported
-        # on SRX loopback stateless filter
-        config.Append(family_keywords['protocol'] +
-                      ' ' + self._Group(self.term.protocol))
-
-      # protocol
-      if self.term.protocol_except:
-        # same as above
-        config.Append(family_keywords['protocol-except'] + ' ' + self._Group(self.term.protocol_except))
-
       # port
       if self.term.port:
         config.Append('port %s' % self._Group(self.term.port))
@@ -501,6 +490,24 @@ class Term(aclgenerator.Term):
       if self.term.ether_type:
         config.Append('ether-type %s' %
                       self._Group(self.term.ether_type))
+      # protocol
+      if self.term.protocol:
+        # both are supported on JunOS, but only icmp6 is supported
+        # on SRX loopback stateless filter, so set all instances of icmpv6 to icmp6.
+        if 'icmpv6' in self.term.protocol:
+          loc = self.term.protocol.index('icmpv6')
+          self.term.protocol[loc] = 'icmp6'
+        config.Append(family_keywords['protocol'] + ' ' +
+                      self._Group(self.term.protocol))
+
+      # protocol
+      if self.term.protocol_except:
+        # same as above
+        if 'icmpv6' in self.term.protocol_except:
+          loc = self.term.protocol_except.index('icmpv6')
+          self.term.protocol_except[loc] = 'icmp6'
+        config.Append(family_keywords['protocol-except'] + ' ' +
+                      self._Group(self.term.protocol_except))
 
       if self.term.traffic_type:
         config.Append('traffic-type %s' %
@@ -566,6 +573,8 @@ class Term(aclgenerator.Term):
       unique_actions.update(self.term.action)
     if self.term.encapsulate:
       unique_actions.add('encapsulate')
+    if self.term.decapsulate:
+      unique_actions.add('decapsulate')
     if len(unique_actions) <= 1:
       for action in [
           self.term.logging, self.term.routing_instance, self.term.counter,
@@ -610,6 +619,10 @@ class Term(aclgenerator.Term):
         config.Append('then {')
         config.Append('encapsulate %s;' % str(self.term.encapsulate))
         config.Append('}')
+      elif current_action == 'decapsulate':
+        config.Append('then {')
+        config.Append('decapsulate %s;' % str(self.term.decapsulate))
+        config.Append('}')
       else:
         config.Append('then %s;' % current_action)
     elif len(unique_actions) > 1:
@@ -655,6 +668,8 @@ class Term(aclgenerator.Term):
           config.Append('next-ip6 %s;' % str(self.term.next_ip[0]))
       if self.term.encapsulate:
         config.Append('encapsulate %s;' % str(self.term.encapsulate))
+      if self.term.decapsulate:
+        config.Append('decapsulate %s;' % str(self.term.decapsulate))
       for action in self.extra_actions:
         config.Append(action + ';')
 
@@ -689,6 +704,8 @@ class Term(aclgenerator.Term):
     action = set(self.term.action)
     if self.term.encapsulate:
       action.add(self.term.encapsulate)
+    if self.term.decapsulate:
+      action.add(self.term.decapsulate)
     if self.term.routing_instance:
       action.add(self.term.routing_instance)
     if len(action) > 1:
@@ -893,6 +910,7 @@ class Juniper(aclgenerator.ACLGenerator):
     supported_tokens |= {'address',
                          'restrict_address_family',
                          'counter',
+                         'decapsulate',
                          'destination_prefix',
                          'destination_prefix_except',
                          'dscp_except',
