@@ -1,4 +1,4 @@
-# Copyright 2022 Google Inc. All Rights Reserved.
+# Copyright 2023 Google Inc. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -133,6 +133,7 @@ class Term(aclgenerator.Term):
     Function is used inside PortsAndProtocols.
 
     Args:
+      af: address family.
       term_icmp_types: ICMP types keywords.
 
     Returns:
@@ -349,7 +350,9 @@ class Term(aclgenerator.Term):
     else:
       return ''
 
-  def GroupExpressions(self, address_expr, pp_expr, options, verdict):
+  def GroupExpressions(
+      self, int_expr, address_expr, pp_expr, options, verdict, comment
+  ):
     """Combines all expressions with a verdict (decision).
 
     The inputs are already pre-sanitized by RulesetGenerator. NFTables processes
@@ -357,11 +360,13 @@ class Term(aclgenerator.Term):
     towards the end append any term.options from _OptionsHandler.
 
     Args:
+      int_expr: RulesetGenerator source or destination interface str.
       address_expr: pre-processed list of nftable statements of network
         addresses.
       pp_expr: pre-processed list of nftables protocols and ports.
       options: string value to append before verdict for NFT special options.
       verdict: action to take on resulting final statement (allow/deny).
+      comment: term.comment string adhering to NFT limits.
 
     Returns:
       list of strings representing valid nftables statements.
@@ -388,7 +393,14 @@ class Term(aclgenerator.Term):
         statement.append(pstat + Add(options) + Add(verdict))
     else:
       # If no addresses or ports & protocol. Verdict only statement.
-      statement.append((Add(options) + verdict))
+      statement.append((Add(options) + Add(verdict)))
+    # source/destination interface handling always to be done at the end.
+    if int_expr:
+      # 'statement' is a list because join to another list in RulesetGenerator.
+      statement[0] = int_expr + Add(statement[0])
+    # Handling of comments should always be done after verdict statement.
+    if comment:
+      statement[0] = statement[0] + Add(comment)
     return statement
 
   def _AddrStatement(self, address_family, src_addr, dst_addr):
@@ -455,12 +467,21 @@ class Term(aclgenerator.Term):
       list of strings. Representing a ruleset for later formatting.
     """
     term_ruleset = []
+    unique_term_ruleset = []
+    comment = ''
 
     # COMMENT handling.
     if self.verbose:
-      term_ruleset.append(
-          'comment ' + aclgenerator.TruncateWords(
-              self.term.comment, Nftables.COMMENT_CHAR_LIMIT))
+      comment = 'comment ' + aclgenerator.TruncateWords(
+          self.term.comment, Nftables.COMMENT_CHAR_LIMIT)
+
+    # INTERFACE (source/destination) handling.
+    if term.source_interface:
+      interface = 'iifname' + Add(term.source_interface)
+    elif term.destination_interface:
+      interface = 'oifname' + Add(term.destination_interface)
+    else:
+      interface = ''
     # OPTIONS / LOGGING / COUNTERS
     opt = self._OptionsHandler(term)
     # STATEMENT VERDICT / ACTION.
@@ -473,6 +494,11 @@ class Term(aclgenerator.Term):
       address_list = self._AddrStatement(address_family,
                                          self.term.source_address,
                                          self.term.destination_address)
+      # Check if we're dealing with a term of a different IP family that needs
+      # to be skipped.
+      if not address_list and (
+          self.term.source_address or self.term.destination_address):
+        continue
 
       # PORTS and PROTOCOLS handling.
       proto_and_ports = self.PortsAndProtocols(address_family,
@@ -480,12 +506,21 @@ class Term(aclgenerator.Term):
                                                self.term.source_port,
                                                self.term.destination_port,
                                                self.term.icmp_type)
+      # Do not render ICMP types if IP family mismatch.
+      if ((address_family == 'ip6' and 'icmp' in self.term.protocol) or
+          (address_family == 'ip' and ('icmpv6' in self.term.protocol)
+           or 'icmp6' in self.term.protocol)):
+        continue
 
       # TODO: If verdict is not supported, drop nftable_rule for it.
-      nftable_rule = self.GroupExpressions(address_list, proto_and_ports, opt,
-                                           verdict)
+      nftable_rule = self.GroupExpressions(
+          interface, address_list, proto_and_ports, opt, verdict, comment
+      )
       term_ruleset.extend(nftable_rule)
-    return term_ruleset
+    # Ensure that chain statements contain no duplicates rules.
+    unique_term_ruleset = [
+        i for n, i in enumerate(term_ruleset) if i not in term_ruleset[:n]]
+    return unique_term_ruleset
 
   def _AddressClassifier(self, address_to_classify):
     """Organizes network addresses according to IP family in a dict.
@@ -593,9 +628,11 @@ class Nftables(aclgenerator.ACLGenerator):
         'protocol',
         'platform',
         'platform_exclude',
+        'source_interface', # NFT iifname
         'source_address',
         'source_address_exclude',
         'source_port',
+        'destination_interface', # NFT oifname
         'translated',  # obj attribute, not token
         'stateless_reply',
     }
@@ -649,11 +686,16 @@ class Nftables(aclgenerator.ACLGenerator):
       child_chains = collections.defaultdict(dict)
       term_names = set()
       new_terms = []
-      # TODO: Add checks for ICMP and address families.
       for term in terms:
         if term.name in term_names:
           raise TermError('Duplicate term name')
         term_names.add(term.name)
+        if term.source_interface and term.destination_interface:
+          raise TermError(
+              'Incorrect interface on term. Must be either be a source or'
+              ' destination, not both.'
+          )
+          continue
         if term.stateless_reply:
           logging.warning(
               'WARNING: Term %s is a stateless reply '
@@ -764,7 +806,7 @@ class Nftables(aclgenerator.ACLGenerator):
     for (header, base_chain_name, nf_af, nf_hook, nf_priority,
          filter_policy_default_action, verbose, child_chains) in nft_pol:
       base_chain_comment = ''
-      # Add max character checking on header.comment later if needed.
+      # TODO: If child_chain ruleset is empty don't store term.
       if verbose:
         base_chain_comment = header.comment
       nftables[nf_af][base_chain_name] = {
@@ -795,20 +837,22 @@ class Nftables(aclgenerator.ACLGenerator):
           # First time we comment it out so .nft file is human-readable.
           nft_config.append(
               TabSpacer(8, '#' + ' '.join(base_chain_dict[item]['comment'])))
-          # Second time so 'nft list ruleset' keeps the comment in memory.
-          nft_config.append(
-              TabSpacer(
-                  8, 'comment ' +
-                  aclgenerator.TruncateWords(
-              base_chain_dict[item]['comment'], self.COMMENT_CHAR_LIMIT)))
         nft_config.append(
             TabSpacer(
                 8, 'type filter hook %s priority %s; policy %s;' %
                 (base_chain_dict[item]['hook'],
                  base_chain_dict[item]['priority'],
                  base_chain_dict[item]['policy'])))
-        # stateful firewall: allow reply traffic.
-        nft_config.append(TabSpacer(8, 'ct state established,related accept'))
+        # Add policy header comment after stateful firewall rule.
+        if base_chain_dict[item]['comment']:
+          nft_config.append(TabSpacer(8, 'ct state established,related accept'
+                                      + Add('comment') +
+                                      Add(aclgenerator.TruncateWords(
+                                          base_chain_dict[item]['comment'],
+                                          self.COMMENT_CHAR_LIMIT))))
+        else:
+          # stateful firewall: allows reply traffic.
+          nft_config.append(TabSpacer(8, 'ct state established,related accept'))
         # Reference the child chains with jump.
         for child_chain in base_chain_dict[item]['rules'][item].keys():
           nft_config.append(TabSpacer(8, 'jump %s' % child_chain))
