@@ -254,7 +254,9 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
   SUFFIX = ".xml"
   _SUPPORTED_AF = set(("inet", "inet6", "mixed"))
   _AF_MAP = {"inet": (4,), "inet6": (6,), "mixed": (4, 6)}
-  _TERM_MAX_LENGTH = 31
+  _TERM_MAX_LENGTH = 63
+  _APPLICATION_NAME_MAX_LENGTH = 31
+  _TERM_PREFIX_LENGTH = 24
   _SUPPORTED_PROTO_NAMES = [
       "tcp",
       "udp",
@@ -423,6 +425,11 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
             '; expect {%s}' % (filter_options[5], '|'.join(valid_addr_obj)))
       no_addr_obj = True if (len(filter_options) > 5 and
                              filter_options[5] == "no-addr-obj") else False
+
+      unique_term_prefixes = True if (
+          len(filter_options) > 6 and
+          filter_options[6] == "unique-term-prefixes") else False
+
       if first_addr_obj is None:
         first_addr_obj = no_addr_obj
       if first_addr_obj != no_addr_obj:
@@ -462,6 +469,13 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
           if self._PLATFORM in term.platform_exclude:
             continue
 
+        if unique_term_prefixes:
+          # Prefix hash of from_zone and to_zone to the term name to get mostly
+          # unique names across different policies.
+          # This is not crytopgraphically guaranteed.
+          # This modified term name should not exceed _TERM_MAX_LENGTH.
+          term.name = self.HexDigest(self.from_zone + self.to_zone,
+                                     self._TERM_PREFIX_LENGTH) + "-" + term.name
         term.name = self.FixTermLength(term.name)
         if term.name in term_dup_check:
           raise PaloAltoFWDuplicateTermError("You have a duplicate term: %s" %
@@ -674,9 +688,11 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
             if icmp_version == "icmp":
               icmp_app_name = "icmp-%s" % term_icmp_type_name
               # This is to abbreviate the Application name where possible.
-              # The limit is defined by _TERM_MAX_LENGTH = 31.
-              if len(icmp_app_name) > self._TERM_MAX_LENGTH:
-                icmp_app_name = self.FixTermLength(icmp_app_name, True, True)
+              # The limit is defined by _APPLICATION_NAME_MAX_LENGTH = 31.
+              if len(icmp_app_name) > self._APPLICATION_NAME_MAX_LENGTH:
+                icmp_app_name = self.FixTermLength(
+                    icmp_app_name, True, True,
+                    self._APPLICATION_NAME_MAX_LENGTH)
               if term_icmp_type_name not in policy.Term.ICMP_TYPE[4]:
                 raise PaloAltoFWBadIcmpTypeError(
                     "term with bad icmp type: %s, icmp_type: %s" %
@@ -685,9 +701,11 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
             else:
               icmp_app_name = "icmp6-%s" % term_icmp_type_name
               # This is to abbreviate the Application name where possible.
-              # The limit is defined by _TERM_MAX_LENGTH = 31.
-              if len(icmp_app_name) > self._TERM_MAX_LENGTH:
-                icmp_app_name = self.FixTermLength(icmp_app_name, True, True)
+              # The limit is defined by _APPLICATION_NAME_MAX_LENGTH = 31.
+              if len(icmp_app_name) > self._APPLICATION_NAME_MAX_LENGTH:
+                icmp_app_name = self.FixTermLength(
+                    icmp_app_name, True, True,
+                    self._APPLICATION_NAME_MAX_LENGTH)
               if term_icmp_type_name not in policy.Term.ICMP_TYPE[6]:
                 raise PaloAltoFWBadIcmpTypeError(
                     "term with bad icmp type: %s, icmp_type: %s" %
@@ -760,14 +778,16 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
     if zone not in self.addressbook:
       self.addressbook[zone] = collections.OrderedDict()
     if address.parent_token not in self.addressbook[zone]:
-      self.addressbook[zone][address.parent_token] = []
+      # Store addresses as keys to enable quick lookups.
+      # key: address, value: name
+      self.addressbook[zone][address.parent_token] = collections.OrderedDict()
     name = address.parent_token
-    for ip in self.addressbook[zone][name]:
-      if str(address) == str(ip[0]):
-        return
+    if address in self.addressbook[zone][name]:
+      return
     counter = len(self.addressbook[zone][address.parent_token])
     name = "%s_%s" % (name, str(counter))
-    self.addressbook[zone][address.parent_token].append((address, name))
+
+    self.addressbook[zone][address.parent_token][address] = name
 
   def _SortAddressBookNumCheck(self, item):
     """Used to give a natural order to the list of acl entries.
@@ -823,7 +843,7 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
       # building individual addresses dictionary
       groups = sorted(self.addressbook[zone])
       for group in groups:
-        for address, name in self.addressbook[zone][group]:
+        for address, name in self.addressbook[zone][group].items():
           if name in address_book_names_dict:
             if address_book_names_dict[name].supernet_of(address):
               continue
@@ -832,7 +852,7 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
         # building individual address-group dictionary
         for nested_group in groups:
           group_names = []
-          for address, name in self.addressbook[zone][nested_group]:
+          for address, name in self.addressbook[zone][nested_group].items():
             group_names.append(name)
           address_book_groups_dict[nested_group] = group_names
 
@@ -884,7 +904,6 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
     etree.SubElement(vsys_entry, "application-group")
 
     # SERVICES
-    vsys_entry.append(etree.Comment(" Services "))
     service = etree.SubElement(vsys_entry, "service")
     for k, v in self.service_map.entries.items():
       entry = etree.SubElement(service, "entry", {"name": v["name"]})
@@ -905,7 +924,6 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
         sport.text = tup.replace("'", "").replace(", ", ",")
 
     # RULES
-    vsys_entry.append(etree.Comment(" Rules "))
     rulebase = etree.SubElement(vsys_entry, "rulebase")
     security = etree.SubElement(rulebase, "security")
     rules = etree.SubElement(security, "rules")
@@ -913,7 +931,6 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
 
     tag_num = 0
 
-    # pytype: disable=key-error
     # pylint: disable=unused-variable
     for (header, pa_rules, filter_options) in self.pafw_policies:
       tag_name = None
@@ -1078,14 +1095,11 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
             log = etree.SubElement(entry, "log-end")
             log.text = "yes"
 
-    # pytype: enable=key-error
-
     if no_addr_obj:
       address_book_groups_dict = {}
       address_book_keys = {}
 
     # ADDRESS
-    vsys_entry.append(etree.Comment(" Address Groups "))
     addr_group = etree.SubElement(vsys_entry, "address-group")
 
     for group, address_list in address_book_groups_dict.items():
@@ -1095,7 +1109,6 @@ class PaloAltoFW(aclgenerator.ACLGenerator):
         member = etree.SubElement(static, "member")
         member.text = name
 
-    vsys_entry.append(etree.Comment(" Addresses "))
     addr = etree.SubElement(vsys_entry, "address")
 
     for name in address_book_keys:
