@@ -15,11 +15,6 @@
 
 """Speedway iptables generator.  This is a subclass of Iptables lib."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 from string import Template
 from capirca.lib import iptables
 
