@@ -476,6 +476,40 @@ term bad-term-2 {
   routing-instance:: instance-name
 }
 """
+DECAPSULATE_GOOD_TERM_1 = """
+term good-term-1 {
+  protocol:: udp
+  decapsulate:: template-name
+}
+"""
+DECAPSULATE_GOOD_TERM_2 = """
+term good-term-2 {
+  protocol:: udp
+  decapsulate:: template-name
+  counter:: count-name
+}
+"""
+DECAPSULATE_BAD_TERM_1 = """
+term bad-term-1 {
+  protocol:: udp
+  decapsulate:: template-name
+  action:: accept
+}
+"""
+DECAPSULATE_BAD_TERM_2 = """
+term bad-term-2 {
+  protocol:: udp
+  decapsulate:: template-name
+  routing-instance:: instance-name
+}
+"""
+DECAPSULATE_BAD_TERM_3 = """
+term bad-term-2 {
+  protocol:: udp
+  decapsulate:: template-name
+  encapsulate:: something
+}
+"""
 PORTMIRROR_GOOD_TERM_1 = """
 term good-term-1 {
   protocol:: tcp
@@ -606,6 +640,7 @@ SUPPORTED_TOKENS = frozenset([
     'dscp_except',
     'dscp_match',
     'dscp_set',
+    'decapsulate',
     'encapsulate',
     'ether_type',
     'expiration',
@@ -814,6 +849,33 @@ class JuniperTest(parameterized.TestCase):
         EXP_INFO)
     self.assertRaises(juniper.JuniperMultipleTerminatingActionError, str, jcl)
 
+  def testDecapsulate(self):
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + DECAPSULATE_GOOD_TERM_1, self.naming),
+        EXP_INFO)
+    output = str(jcl)
+    self.assertIn('decapsulate template-name;', output, output)
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + DECAPSULATE_GOOD_TERM_2, self.naming),
+        EXP_INFO)
+    output = str(jcl)
+    self.assertIn('decapsulate template-name;', output, output)
+    self.assertIn('count count-name;', output, output)
+
+  def testFailDecapsulate(self):
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + DECAPSULATE_BAD_TERM_1, self.naming),
+        EXP_INFO)
+    self.assertRaises(juniper.JuniperMultipleTerminatingActionError, str, jcl)
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + DECAPSULATE_BAD_TERM_2, self.naming),
+        EXP_INFO)
+    self.assertRaises(juniper.JuniperMultipleTerminatingActionError, str, jcl)
+    jcl = juniper.Juniper(
+        policy.ParsePolicy(GOOD_HEADER + DECAPSULATE_BAD_TERM_3, self.naming),
+        EXP_INFO)
+    self.assertRaises(juniper.JuniperMultipleTerminatingActionError, str, jcl)
+
   def testPortMirror(self):
     jcl = juniper.Juniper(
         policy.ParsePolicy(GOOD_HEADER + PORTMIRROR_GOOD_TERM_1, self.naming),
@@ -861,7 +923,7 @@ class JuniperTest(parameterized.TestCase):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER_V6 + GOOD_TERM_1_V6,
                                              self.naming), EXP_INFO)
     output = str(jcl)
-    self.assertTrue('next-header icmpv6;' in output and
+    self.assertTrue('next-header icmp6;' in output and
                     'next-header tcp;' in output, output)
 
     self.naming.GetNetAddr.assert_called_once_with('SOME_HOST')
@@ -929,7 +991,7 @@ class JuniperTest(parameterized.TestCase):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER_V6 + GOOD_TERM_20_V6,
                                              self.naming), EXP_INFO)
     output = str(jcl)
-    self.assertIn('next-header-except icmpv6;', output, output)
+    self.assertIn('next-header-except icmp6;', output, output)
 
   def testProtocolCase(self):
     jcl = juniper.Juniper(policy.ParsePolicy(GOOD_HEADER + GOOD_TERM_5,
