@@ -216,6 +216,19 @@ term good-term-5 {
 }
 """
 
+GOOD_TERM_COUNT_1 = """
+term good-term-6 {
+  counter:: good-counter
+  action:: accept
+}
+"""
+GOOD_TERM_COUNT_2 = """
+term good-term-6 {
+  counter:: good-counter
+  action:: deny
+}
+"""
+
 GOOD_TERM_10 = """
 term good-term-10 {
   destination-address:: SOME_HOST
@@ -506,6 +519,16 @@ term timeout-term {
 }
 """
 
+GLOBAL_ZONE_TERM = """
+term global-zone-term {
+  protocol:: icmp
+  icmp-type:: echo-request
+  source-zone:: szone2 szone1
+  destination-zone:: dzone2 dzone1
+  action:: accept
+}
+"""
+
 PLATFORM_EXCLUDE_TERM = """
 term platform-exclude-term {
   protocol:: tcp udp
@@ -531,15 +554,18 @@ term platform-exclude-term {
 }
 """
 
-SUPPORTED_TOKENS = {
+SUPPORTED_TOKENS = frozenset({
     'action',
     'comment',
+    'counter',
     'destination_address',
     'destination_address_exclude',
     'destination_port',
+    'destination_zone',
     'dscp_except',
     'dscp_match',
     'dscp_set',
+    'source_zone',
     'expiration',
     'icmp_type',
     'stateless_reply',
@@ -557,7 +583,7 @@ SUPPORTED_TOKENS = {
     'translated',
     'verbatim',
     'vpn'
-}
+})
 
 SUPPORTED_SUB_TOKENS = {
     'action': {'accept', 'deny', 'reject', 'count', 'log', 'dscp'},
@@ -811,6 +837,20 @@ class JuniperSRXTest(absltest.TestCase):
     self.assertIn('session-init;', output)
     self.assertNotIn('session-close;', output)
 
+  def testCounterAccept(self):
+    srx = junipersrx.JuniperSRX(policy.ParsePolicy(GOOD_HEADER
+                                                   + GOOD_TERM_COUNT_1,
+                                                   self.naming), EXP_INFO)
+    output = str(srx)
+    self.assertIn('count;', output)
+
+  def testCounterDeny(self):
+    srx = junipersrx.JuniperSRX(policy.ParsePolicy(GOOD_HEADER
+                                                   + GOOD_TERM_COUNT_2,
+                                                   self.naming), EXP_INFO)
+    output = str(srx)
+    self.assertIn('count;', output)
+
   def testOwnerTerm(self):
     pol = policy.ParsePolicy(GOOD_HEADER + OWNER_TERM, self.naming)
     output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
@@ -911,6 +951,19 @@ class JuniperSRXTest(absltest.TestCase):
     output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
     self.assertIn('timeout 77', output, output)
 
+  def testZoneGlobal(self):
+    pol = policy.ParsePolicy(GOOD_HEADER_10 + GLOBAL_ZONE_TERM, self.naming)
+    output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
+    self.assertIn('from-zone', output, output)
+    self.assertIn('szone1 szone2', output, output)
+    self.assertIn('to-zone', output, output)
+    self.assertIn('dzone1 dzone2', output, output)
+
+  def testZoneNonGlobal(self):
+    pol = policy.ParsePolicy(GOOD_HEADER + GLOBAL_ZONE_TERM, self.naming)
+    self.assertRaises(junipersrx.UnsupportedFilterError,
+                      junipersrx.JuniperSRX, pol, EXP_INFO)
+
   def testIcmpV6(self):
     pol = policy.ParsePolicy(GOOD_HEADER + IPV6_ICMP_TERM, self.naming)
     output = str(junipersrx.JuniperSRX(pol, EXP_INFO))
