@@ -13,12 +13,8 @@
 # limitations under the License.
 """UnitTest class for nsxv.py."""
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
-import unittest
+from absl.testing import absltest
+from unittest import mock
 from xml.etree import ElementTree as ET
 
 from capirca.lib import nacaddr
@@ -26,8 +22,6 @@ from capirca.lib import naming
 from capirca.lib import nsxv
 from capirca.lib import policy
 
-import mock
-
 
 INET_TERM = """\
   term permit-mail-services {
@@ -398,10 +392,10 @@ EXP_INFO = 2
 _PLATFORM = 'nsxv'
 
 
-class TermTest(unittest.TestCase):
+class TermTest(absltest.TestCase):
 
   def setUp(self):
-    super(TermTest, self).setUp()
+    super().setUp()
     self.naming = mock.create_autospec(naming.Naming)
 
   def testInitForinet(self):
@@ -997,4 +991,4 @@ class TermTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
-  unittest.main()
+  absltest.main()
