@@ -91,15 +91,40 @@ header {
 """
 
 INCLUDE_STATEMENT = """
-#include "/tmp/y.inc"
+#include "includes/y.inc"
 """
 INCLUDED_Y_FILE = """
 term included-term-1 {
   protocol:: tcp
   action:: accept
 }
+#include "includes/z.inc"
+"""
+
+BAD_INCLUDED_FILE = """
+term included-term-1 {
+  protocol:: tcp
+  action:: accept
+}
 #include "/tmp/z.inc"
 """
+
+BAD_INCLUDED_FILE_1 = """
+term included-term-1 {
+  protocol:: tcp
+  action:: accept
+}
+#include "includes/../../etc/passwd.inc"
+"""
+
+GOOD_INCLUDED_FILE_1 = """
+term good-included-term-1 {
+  protocol:: tcp
+  action:: accept
+}
+#include "includes/../pol/z.inc"
+"""
+
 GOOD_TERM_0 = """
 term good-term-0 {
   protocol:: icmp
@@ -431,6 +456,27 @@ term good-term-47 {
   port-mirror:: true
 }
 """
+GOOD_TERM_48 = """
+term good-term-48 {
+  protocol:: icmp
+  source-zone:: zone1 zone2
+  destination-zone:: zone1 zone2
+  action:: accept
+}
+"""
+GOOD_TERM_49 = """
+term good-term-46 {
+  protocol:: udp
+  decapsulate:: mpls-in-udp
+}
+"""
+GOOD_TERM_50 = """
+term good-term-45 {
+  source-address:: ANY
+  action:: accept
+  source-service-accounts:: acct1@blah.com
+}
+"""
 GOOD_TERM_V6_1 = """
 term good-term-v6-1 {
   hop-limit:: 5
@@ -604,9 +650,46 @@ class PolicyTest(absltest.TestCase):
     # ensure good-term-1 shows up as the second term
     self.assertEqual(terms[2].name, 'good-term-1')
 
-    mock_file.assert_has_calls([
-        mock.call('/tmp/y.inc'),
-        mock.call('/tmp/z.inc')])
+    mock_file.assert_has_calls(
+        [mock.call('includes/y.inc'), mock.call('includes/z.inc')]
+    )
+
+  @mock.patch.object(policy, '_ReadFile')
+  def testBadIncludes(self, mock_file):
+    """Ensure nested includes error handling works."""
+    mock_file.side_effect = [BAD_INCLUDED_FILE, GOOD_TERM_5]
+
+    # contents of our base policy (which has a bad included file)
+    pol = HEADER + INCLUDE_STATEMENT + GOOD_TERM_1
+    self.assertRaises(
+        policy.InvalidIncludeDirectoryError,
+        policy.ParsePolicy,
+        pol,
+        self.naming,
+    )
+    # Ensuring relative paths don't bypass invalid directory checks
+    mock_file.side_effect = [BAD_INCLUDED_FILE_1, GOOD_TERM_5]
+    pol = HEADER + BAD_INCLUDED_FILE_1 + GOOD_TERM_1
+    self.assertRaises(
+        policy.InvalidIncludeDirectoryError,
+        policy.ParsePolicy,
+        pol,
+        self.naming,
+    )
+
+  @mock.patch.object(policy, '_ReadFile')
+  def testGoodIncludesWithRelativePaths(self, mock_file):
+    """Ensure nested includes error handling works for valid files."""
+    mock_file.side_effect = [GOOD_TERM_5]
+    # base policy has a good included file, with relative paths
+    pol = HEADER + GOOD_INCLUDED_FILE_1 + GOOD_TERM_1
+    p = policy.ParsePolicy(pol, self.naming)
+    _, terms = p.filters[0]
+    # ensure include worked and we now have 3 terms in this policy
+    self.assertEqual(len(terms), 3)
+    self.assertEqual(terms[0].name, 'good-included-term-1')
+    self.assertEqual(terms[1].name, 'good-term-5')
+    self.assertEqual(terms[2].name, 'good-term-1')
 
   def testGoodPol(self):
     pol = HEADER + GOOD_TERM_1 + GOOD_TERM_2
@@ -1190,7 +1273,7 @@ class PolicyTest(absltest.TestCase):
     self.assertEqual(8, term.AddressesByteLength([6]))
     self.assertEqual(10, term.AddressesByteLength())
 
-# pylint: enable=maybe-no-member
+  # pylint: enable=maybe-no-member
 
   def testICMPCodes(self):
     pol = HEADER + GOOD_TERM_42
@@ -1237,11 +1320,25 @@ class PolicyTest(absltest.TestCase):
     result = policy.ParsePolicy(pol, self.naming)
     self.assertIn('encapsulate: stuff_and_things', str(result))
 
+  def testDecapsulate(self):
+    pol = HEADER + GOOD_TERM_49
+    result = policy.ParsePolicy(pol, self.naming)
+    self.assertIn('decapsulate: mpls-in-udp', str(result))
+
   def testPortMirror(self):
     pol = HEADER + GOOD_TERM_47
     result = policy.ParsePolicy(pol, self.naming)
     self.assertIn('port_mirror: true', str(result))
 
+  def testSrxGLobalZone(self):
+    pol = HEADER + GOOD_TERM_48
+    result = policy.ParsePolicy(pol, self.naming)
+    zones = ['zone1', 'zone2']
+    expected_source = 'source_zone: %s' % zones
+    expected_destination = 'destination_zone: %s' % zones
+    self.assertIn(expected_source, str(result))
+    self.assertIn(expected_destination, str(result))
+
   def testTTL(self):
     pol = HEADER + GOOD_TERM_43
     result = policy.ParsePolicy(pol, self.naming)
@@ -1286,6 +1383,15 @@ class PolicyTest(absltest.TestCase):
     self.assertRaises(policy.MixedPortandNonPortProtos, policy.ParsePolicy,
                       pol, self.naming)
 
+  def testSourceServiceAccount(self):
+    pol = HEADER_HF_1 + GOOD_TERM_50
+
+    result = policy.ParsePolicy(pol, self.naming)
+    term = result.filters[0][1][0]
+    self.assertEqual(
+        ['acct1@blah.com'],
+        term.source_service_accounts)
+
   def testTargetServiceAccount(self):
     pol = HEADER_HF_1 + GOOD_TERM_45
 
